@@ -9,7 +9,6 @@ import sys
 from abc import ABC, abstractmethod
 from asyncio import TimeoutError
 from functools import lru_cache, reduce
-from glob import iglob
 from operator import iconcat
 from os import (
     chdir,
@@ -26,7 +25,7 @@ from os import (
 from pathlib import Path
 from platform import system
 from shlex import quote as _quote, split as _shplit
-from shutil import move, which as _which
+from shutil import which as _which
 from subprocess import (
     DEVNULL,
     PIPE,
@@ -65,7 +64,8 @@ from shellfish.fs import (
 )
 from shellfish.process import is_win
 from shellfish.sh._dirtree import _DirTree
-from xtyping import STDIN, FsPath, IterableStr, TypedDict
+from shellfish.sp import PopenArgs
+from xtyping import STDIN, AnyStr, FsPath, IterableStr, TypedDict
 
 __all__ = (
     "Done",
@@ -129,8 +129,6 @@ __all__ = (
 
 IS_WIN: bool = is_win()
 
-PopenArgs = Union[bytes, str, Sequence[str], Sequence[FsPath]]
-
 
 class FlagMeta(type):
     """Meta class"""
@@ -308,22 +306,26 @@ class Done(JsonBaseModel):
         """Returns a CalledProcessError object"""
         return DoneError(done=self)
 
-    def check(self) -> None:
+    def check(self, ok_code: Union[int, Sequence[int]] = 0) -> None:
         """Check returncode and stderr
 
         Raises:
             DoneError: If return code is non-zero and stderr is not None
 
         """
-        if self.returncode and self.stderr:
-            raise DoneError(done=self)
+        if isinstance(ok_code, int):
+            if self.returncode != ok_code and self.stderr:
+                raise DoneError(done=self)
+        else:
+            if self.returncode not in ok_code:
+                raise DoneError(done=self)
 
     def sys_print(self) -> None:
         """Write self.stdout to sys.stdout and self.stderr to sys.stderr"""
         sys.stdout.write(self.stdout)
         sys.stderr.write(self.stderr)
 
-    def write_stdout(self, filepath: str, *, append: bool = False) -> None:
+    def write_stdout(self, filepath: FsPath, *, append: bool = False) -> None:
         """Write stdout as a string to a fspath
 
         Args:
@@ -333,7 +335,7 @@ class Done(JsonBaseModel):
         """
         fs.wstring(Path(filepath), self.stdout, append=append)
 
-    def write_stderr(self, filepath: str, *, append: bool = False) -> None:
+    def write_stderr(self, filepath: FsPath, *, append: bool = False) -> None:
         """Write stderr as a string to a fspath
 
         Args:
@@ -343,7 +345,7 @@ class Done(JsonBaseModel):
         """
         fs.wstring(Path(filepath), self.stderr, append=append)
 
-    def __gt__(self, filepath: str) -> None:  # type: ignore
+    def __gt__(self, filepath: FsPath) -> None:
         """Operator overload for writing a stdout to a fspath
 
         Args:
@@ -352,7 +354,7 @@ class Done(JsonBaseModel):
         """
         self.write_stdout(filepath)
 
-    def __ge__(self, filepath: str) -> "Done":
+    def __ge__(self, filepath: FsPath) -> "Done":
         """Operator overload for writing stderr to fspath
 
         Args:
@@ -365,7 +367,7 @@ class Done(JsonBaseModel):
         self.write_stderr(filepath)
         return self
 
-    def __rshift__(self, filepath: str) -> None:
+    def __rshift__(self, filepath: FsPath) -> None:
         """Operator overload for appending stdout to fspath
 
         Args:
@@ -374,7 +376,7 @@ class Done(JsonBaseModel):
         """
         self.write_stdout(filepath, append=True)
 
-    def __irshift__(self, filepath: str) -> "Done":
+    def __irshift__(self, filepath: FsPath) -> "Done":
         """Operator overload for appending stderr to fspath
 
         Args:
@@ -403,7 +405,7 @@ class Done(JsonBaseModel):
         ]
 
 
-def decode_stdio_bytes(stdio_bytes: bytes, lf: bool = True) -> str:
+def decode_stdio_bytes(stdio_bytes: AnyStr, lf: bool = True) -> str:
     """Return Stdio bytes from stdout/stderr as a string
 
     Args:
@@ -414,6 +416,8 @@ def decode_stdio_bytes(stdio_bytes: bytes, lf: bool = True) -> str:
         str: decoded stdio bytes
 
     """
+    if isinstance(stdio_bytes, str):
+        return stdio_bytes
     if lf:
         return decode_stdio_bytes(stdio_bytes, lf=False).replace("\r\n", "\n")
     try:
@@ -430,33 +434,37 @@ def decode_stdio_bytes(stdio_bytes: bytes, lf: bool = True) -> str:
     return str(stdio_bytes.decode("latin-1"))
 
 
-def pstdout(proc: CompletedProcess) -> str:
+def pstdout(proc: CompletedProcess[AnyStr]) -> str:
     """Get the STDOUT as a string from a subprocess
 
     Args:
         proc: python subprocess.process object with stdout
 
     Returns:
-        STDOUT for the proc as as string
+        STDOUT for the proc as string
 
     """
-    return decode_stdio_bytes(proc.stdout or b"")
+    if proc.stdout is None:
+        return ""
+    return decode_stdio_bytes(proc.stdout)
 
 
-def pstderr(proc: CompletedProcess) -> str:
+def pstderr(proc: CompletedProcess[AnyStr]) -> str:
     """Get the STDERR as a string from a subprocess
 
     Args:
         proc: python subprocess.process object with STDERR
 
     Returns:
-        STDERR for the proc as as string
+        STDERR for the proc as string
 
     """
-    return decode_stdio_bytes(proc.stderr or b"")
+    if proc.stderr is None:
+        return ""
+    return decode_stdio_bytes(proc.stderr)
 
 
-def pstdout_pstderr(proc: CompletedProcess) -> Tuple[str, str]:
+def pstdout_pstderr(proc: CompletedProcess[AnyStr]) -> Tuple[str, str]:
     """Get the STDOUT and STDERR as strings from a subprocess
 
     Args:
@@ -554,6 +562,7 @@ def _do(
     input: STDIN = None,
     timeout: Optional[int] = None,
     text: bool = False,
+    ok_code: Union[int, Sequence[int]] = 0,
 ) -> Done:
     """Run a subprocess synchronously
 
@@ -620,8 +629,8 @@ def _do(
         verbose=verbose,
         stdin=_input,
     )
-    if check:
-        done.check()
+    if check or ok_code != 0:
+        done.check(ok_code=ok_code)
     return done
 
 
@@ -636,6 +645,7 @@ def do(
     verbose: bool = False,
     input: STDIN = None,
     timeout: Optional[int] = None,
+    ok_code: Union[int, Sequence[int]] = 0,
 ) -> Done:
     """Run a subprocess synchronously
 
@@ -651,6 +661,7 @@ def do(
         verbose (bool): Flag to write the subprocess stdout and stderr to
             sys.stdout and sys.stderr
         timeout (Optional[int]): Timeout in seconds for the process if not None
+        ok_code: Return code(s) to check against
 
     Returns:
         Finished PRun object which is a dictionary, so a dictionary
@@ -675,6 +686,7 @@ def do(
         verbose=verbose,
         input=_input,
         timeout=timeout,
+        ok_code=ok_code,
     )
 
 
@@ -688,6 +700,7 @@ def shx(
     verbose: bool = False,
     input: STDIN = None,
     timeout: Optional[int] = None,
+    ok_code: Union[int, Sequence[int]] = 0,
 ) -> Done:
     """Run a subprocess synchronously in current shell
 
@@ -718,6 +731,7 @@ def shx(
         verbose=verbose,
         input=input,
         timeout=timeout,
+        ok_code=ok_code,
     )
 
 
@@ -736,21 +750,41 @@ _do_asyncify = asyncify(do)
 
 
 async def run_async(
-    *popenargs: PopenArgs,
+    args: PopenArgs,
+    *,
+    stdin: Optional[Union[IO[AnyStr], int]] = None,
     input: Optional[str] = None,
+    stdout: Optional[Union[IO[AnyStr], int]] = None,
+    stderr: Optional[Union[IO[AnyStr], int]] = None,
     capture_output: bool = False,
+    shell: bool = False,
+    cwd: Optional[FsPath] = None,
     timeout: Optional[int] = None,
     check: bool = False,
-    **kwargs: Any,
-) -> CompletedProcess:
-    args = validate_popen_args(popenargs)
+    encoding: Optional[str] = None,
+    errors: Optional[str] = None,
+    text: bool = False,
+    env: Optional[Dict[str, str]] = None,
+    universal_newlines: bool = False,
+    **other_popen_kwargs: Any,
+) -> CompletedProcess[Any]:
+    args = validate_popen_args(args)
     complete_subprocess = await _run_async(
         args=args,
         input=input,
-        capture_output=capture_output,
+        stdin=stdin,
+        stdout=stdout,
+        stderr=stderr,
+        shell=shell,
+        cwd=cwd,
         timeout=timeout,
         check=check,
-        **kwargs,
+        errors=errors,
+        env=env,
+        capture_output=capture_output,
+        universal_newlines=universal_newlines | text,
+        encoding=encoding,
+        **other_popen_kwargs,
     )
     return complete_subprocess
 
@@ -767,6 +801,7 @@ async def do_asyncify(
     check: bool = False,
     loop: Optional[Any] = None,
     timeout: Optional[int] = None,
+    ok_code: Union[int, Sequence[int]] = 0,
 ) -> Done:
     """Run a subprocess asynchronously using asyncified version of do"""
     done = await _do_asyncify(  # type: ignore[call-arg]
@@ -780,6 +815,7 @@ async def do_asyncify(
         check=check,
         loop=loop,
         timeout=timeout,
+        ok_code=ok_code,
     )
     done.async_proc = True
     return done
@@ -797,6 +833,7 @@ async def _do_async(
     check: bool = False,
     loop: Optional[Any] = None,
     timeout: Optional[int] = None,
+    ok_code: Union[int, Sequence[int]] = 0,
 ) -> Done:
     """Run a subprocess and await completion
 
@@ -821,9 +858,9 @@ async def _do_async(
         TimeoutError: If the process takes longer than timeout if given
 
     """
-    # if is windows and python is below 3.7, use the asyncified-do func:w
+    # if is windows and python is below 3.7, use the asyncified-do func
     if is_win() and sys.version_info < (3, 8):
-        done = await do_asyncify(  # type: ignore[call-arg]
+        done = await do_asyncify(
             args=args,
             env=env,
             cwd=cwd,
@@ -869,87 +906,78 @@ async def _do_async(
         if exe_path:
             _args[0] = exe_path
 
-    try:
-        if shell:
-            _cmd = args2cmd(_args)
-            ti = time()
-            _proc = await asyncio.create_subprocess_shell(
-                cmd=_cmd,
-                stdin=_stdin,
-                stdout=asyncio.subprocess.PIPE,
-                stderr=asyncio.subprocess.PIPE,
-                env=_env,
-                shell=True,
-                limit=_default_asyncio_stream_limit,
-                cwd=_cwd,
-            )
-        else:
-            ti = time()
-            _proc = await asyncio.create_subprocess_exec(
-                *_args,
-                stdin=_stdin,
-                stdout=asyncio.subprocess.PIPE,
-                stderr=asyncio.subprocess.PIPE,
-                env=_env,
-                limit=_default_asyncio_stream_limit,
-                cwd=_cwd,
-            )
-
-        if timeout:
-            try:
-                (stdout, stderr) = await asyncio.wait_for(
-                    _proc.communicate(input=_input),  # wait for subprocess to finish
-                    timeout=timeout,
-                )
-                tf = time()
-            except TimeoutError as te:
-                _proc.terminate()
-                raise TimeoutError(
-                    str(
-                        {
-                            "args": args,
-                            "input": input,
-                            "env": env,
-                            "cwd": cwd,
-                            "shell": shell,
-                            "asyncio.TimeoutError": str(te),
-                        }
-                    )
-                )
-
-        else:
-            (stdout, stderr) = await _proc.communicate(input=_input)  # wait fo
-            tf = time()
-        if _proc.returncode and stderr and check:
-            raise CalledProcessError(
-                returncode=_proc.returncode, output=stdout, stderr=stderr, cmd=str(args)
-            )
-        return Done(
-            args=args,
-            returncode=_proc.returncode,
-            stdout=decode_stdio_bytes(stdout),
-            stderr=decode_stdio_bytes(stderr),
-            stdin=input,
-            ti=ti,
-            tf=tf,
-            dt=tf - ti,
-            hrdt=HrTime.from_seconds(tf - ti),
-            verbose=verbose,
-            async_proc=True,
+    if shell:
+        _cmd = args2cmd(_args)
+        ti = time()
+        _proc = await asyncio.create_subprocess_shell(
+            cmd=_cmd,
+            stdin=_stdin,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.PIPE,
+            env=_env,
+            shell=True,
+            limit=_default_asyncio_stream_limit,
+            cwd=_cwd,
+        )
+    else:
+        ti = time()
+        _proc = await asyncio.create_subprocess_exec(
+            *_args,
+            stdin=_stdin,
+            stdout=asyncio.subprocess.PIPE,
+            stderr=asyncio.subprocess.PIPE,
+            env=_env,
+            limit=_default_asyncio_stream_limit,
+            cwd=_cwd,
         )
 
-    except NotImplementedError:  # windows python 3.7
-        ...
+    if timeout:
+        try:
+            (stdout, stderr) = await asyncio.wait_for(
+                _proc.communicate(input=_input),  # wait for subprocess to finish
+                timeout=timeout,
+            )
+            tf = time()
+        except TimeoutError as te:
+            _proc.terminate()
+            raise TimeoutError(
+                str(
+                    {
+                        "args": args,
+                        "input": input,
+                        "env": env,
+                        "cwd": cwd,
+                        "shell": shell,
+                        "asyncio.TimeoutError": str(te),
+                    }
+                )
+            )
 
-    return do(
-        args,
-        env=env,
-        extenv=extenv,
-        cwd=cwd,
-        shell=shell,
+    else:
+        (stdout, stderr) = await _proc.communicate(input=_input)  # wait fo
+        tf = time()
+
+    if check or ok_code != 0:
+        _ok_codes = {ok_code} if isinstance(ok_code, int) else set(ok_code)
+        if _proc.returncode and _proc.returncode not in _ok_codes:
+            raise CalledProcessError(
+                returncode=_proc.returncode,
+                output=stdout,
+                stderr=stderr,
+                cmd=str(args),
+            )
+    return Done(
+        args=args,
+        returncode=_proc.returncode,
+        stdout=decode_stdio_bytes(stdout),
+        stderr=decode_stdio_bytes(stderr),
+        stdin=input.decode(encoding="utf-8") if isinstance(input, bytes) else None,
+        ti=ti,
+        tf=tf,
+        dt=tf - ti,
+        hrdt=HrTime.from_seconds(tf - ti),
         verbose=verbose,
-        input=input,
-        timeout=timeout,
+        async_proc=True,
     )
 
 
@@ -993,7 +1021,7 @@ async def do_async(
         raise ValueError("Cannot give *args and args-keyword-argument")
     args = validate_popen_args([*args]) if args else validate_popen_args(popenargs)
     if is_win() and sys.version_info < (3, 8):
-        done = await do_asyncify(  # type: ignore[call-arg]
+        done = await do_asyncify(
             args=args,
             env=env,
             extenv=extenv,
@@ -1219,7 +1247,7 @@ class LIN(OSABC):
             dest = f"{dest}/"
         if not src.endswith("/"):
             src = f"{src}/"
-        _args: List[Union[str, None]] = [  # type: ignore
+        _args: List[Union[str, None]] = [
             "rsync",
             "-a",
             "-O",
@@ -1233,7 +1261,7 @@ class LIN(OSABC):
             src,
             dest,
         ]
-        return list(filter(None, _args))  # type: ignore
+        return list(filter(None, _args))
 
     @staticmethod
     def rsync(
@@ -1748,7 +1776,8 @@ class WIN(OSABC):
         link = str(link)
         target = str(target)
         try:
-            assert path.exists(target) and path.isdir(target)
+            assert path.exists(target), "Target does not exist: {}".format(target)
+            assert path.isdir(target), "Target is not a directory: {}".format(target)
             makedirs(path.split(link)[0], exist_ok=True)
         except Exception:
             ...
@@ -2116,7 +2145,7 @@ def rm(
 
     """
     if verbose or v:
-        exhaust(map(echo, fs.rm_gen(fspath, recursive=recursive, dryrun=dryrun)))
+        exhaust(map(echo, fs.rm_gen(fspath, recursive=recursive or r, dryrun=dryrun)))
         return None
     fs.rm(
         fspath,
@@ -2125,17 +2154,15 @@ def rm(
     )
 
 
-def mv(src: FsPath, dst: FsPath) -> None:
+def mv(src: FsPath, dest: FsPath) -> None:
     """Move file(s) like on the command line
 
     Args:
         src (FsPath): source file(s)
-        dst (FsPath): destination
+        dest (FsPath): destination path
 
     """
-    _dst_str = str(dst)
-    for file in iglob(str(src), recursive=True):
-        move(file, _dst_str)
+    fs.move(src, dest)
 
 
 def source(filepath: FsPath, _globals: bool = True) -> None:
